@@ -6,6 +6,7 @@ from datalad.api import containers_add
 from datalad.api import containers_remove
 from datalad.api import containers_list
 
+from datalad.utils import swallow_outputs
 from datalad.tests.utils import SkipTest
 from datalad.tests.utils import ok_clean_git
 from datalad.tests.utils import with_tree
@@ -16,7 +17,9 @@ from datalad.tests.utils import assert_status
 from datalad.tests.utils import assert_raises
 from datalad.tests.utils import assert_result_count
 from datalad.tests.utils import assert_in
+from datalad.tests.utils import assert_in_results
 from datalad.tests.utils import assert_not_in
+from datalad.tests.utils import assert_re_in
 from datalad.tests.utils import with_tempfile
 from datalad.tests.utils import serve_path_via_http
 from datalad.support.network import get_local_file_url
@@ -38,8 +41,11 @@ def test_add_noop(path):
     ok_clean_git(ds.path)
     # config will be added, as long as there is a file, even when URL access
     # fails
-    res = ds.containers_add('broken', url='bogus', image='dummy',
-                            on_failure='ignore')
+    res = ds.containers_add(
+        'broken',
+        url='bogus-protocol://bogus-server', image='dummy',
+        on_failure='ignore'
+    )
     assert_status('ok', res)
     assert_result_count(res, 1, action='save', status='ok')
 
@@ -65,6 +71,12 @@ def test_add_local_path(path, local_file):
     assert_in(ds.repo.WEB_UUID, ds.repo.whereis(bar_target))
 
 
+RAW_KWDS = dict(return_type='list',
+                result_filter=None,
+                result_renderer=None,
+                result_xfm=None)
+
+
 @with_tempfile
 @with_tree(tree={'some_container.img': "doesn't matter"})
 @serve_path_via_http
@@ -85,7 +97,7 @@ def test_container_files(ds_path, local_file, url):
     ds.save(message="Configure container mountpoint")
 
     # no containers yet:
-    res = ds.containers_list()
+    res = ds.containers_list(**RAW_KWDS)
     assert_result_count(res, 0)
 
     # add first "image": must end up at the configured default location
@@ -98,7 +110,7 @@ def test_container_files(ds_path, local_file, url):
                         action="containers_add")
     ok_(op.lexists(target_path))
 
-    res = ds.containers_list()
+    res = ds.containers_list(**RAW_KWDS)
     assert_result_count(res, 1)
     assert_result_count(
         res, 1,
@@ -110,7 +122,7 @@ def test_container_files(ds_path, local_file, url):
     assert_raises(TypeError, ds.containers_remove)
     res = ds.containers_remove('first', remove_image=True)
     assert_status('ok', res)
-    assert_result_count(ds.containers_list(), 0)
+    assert_result_count(ds.containers_list(**RAW_KWDS), 0)
     # image removed
     assert(not op.lexists(target_path))
 
@@ -150,39 +162,51 @@ def test_container_update(ds_path, local_file, url):
     assert_result_count(res, 1, action="save", status="ok")
     ok_file_has_content(op.join(ds.path, img), "bar")
 
+    # Test commit message
+    # In the above case it was updating existing image so should have "Update "
+    get_commit_msg = lambda *args: ds.repo.format_commit('%B')
+    assert_in("Update ", get_commit_msg())
+
+    # If we add a new image with update=True should say Configure
+    res = ds.containers_add(name="foo2", update=True, url=url_bar)
+    assert_in("Configure ", get_commit_msg())
+
 
 @with_tempfile
 @with_tempfile
 @with_tree(tree={'some_container.img': "doesn't matter"})
-def test_container_from_subdataset(ds_path, subds_path, local_file):
+def test_container_from_subdataset(ds_path, src_subds_path, local_file):
 
     # prepare a to-be subdataset with a registered container
-    subds = Dataset(subds_path).create()
-    subds.containers_add(name="first",
+    src_subds = Dataset(src_subds_path).create()
+    src_subds.containers_add(name="first",
                          url=get_local_file_url(op.join(local_file,
                                                         'some_container.img'))
                          )
     # add it as subdataset to a super ds:
     ds = Dataset(ds_path).create()
-    ds.install("sub", source=subds_path)
+    subds = ds.install("sub", source=src_subds_path)
     # add it again one level down to see actual recursion:
-    Dataset(op.join(ds.path, "sub")).install("subsub", source=subds_path)
+    subds.install("subsub", source=src_subds_path)
 
     # We come up empty without recursive:
-    res = ds.containers_list(recursive=False)
+    res = ds.containers_list(recursive=False, **RAW_KWDS)
     assert_result_count(res, 0)
 
     # query available containers from within super:
-    res = ds.containers_list(recursive=True)
+    res = ds.containers_list(recursive=True, **RAW_KWDS)
     assert_result_count(res, 2)
+    assert_in_results(res, action="containers", refds=ds.path)
 
     # default location within the subdataset:
-    target_path = op.join(ds_path, 'sub',
+    target_path = op.join(subds.path,
                           '.datalad', 'environments', 'first', 'image')
     assert_result_count(
         res, 1,
         name='sub/first', type='file', action='containers', status='ok',
-        path=target_path)
+        path=target_path,
+        parentds=subds.path
+    )
 
     # not installed subdataset doesn't pose an issue:
     sub2 = ds.create("sub2")
@@ -193,12 +217,22 @@ def test_container_from_subdataset(ds_path, subds_path, local_file):
 
     # same results as before, not crashing or somehow confused by a not present
     # subds:
-    res = ds.containers_list(recursive=True)
+    res = ds.containers_list(recursive=True, **RAW_KWDS)
     assert_result_count(res, 2)
-    # default location within the subdataset:
-    target_path = op.join(ds_path, 'sub',
-                          '.datalad', 'environments', 'first', 'image')
     assert_result_count(
         res, 1,
         name='sub/first', type='file', action='containers', status='ok',
-        path=target_path)
+        path=target_path,
+        parentds=subds.path
+    )
+
+    # The default renderer includes the image names.
+    with swallow_outputs() as out:
+        ds.containers_list(recursive=True)
+        lines = out.out.splitlines()
+    assert_re_in("sub/first", lines)
+    assert_re_in("sub/subsub/first", lines)
+    # But we are careful not to render partial names from subdataset traversals
+    # (i.e. we recurse with containers_list(..., result_renderer=None)).
+    with assert_raises(AssertionError):
+        assert_re_in("subsub/first", lines)
